@@ -38,7 +38,7 @@ class EnvironementConfig:
         self.S3_DUMP_BUCKET = base_env_vars.get("S3_DUMP_BUCKET")
         self.VERBOSE = bool(base_env_vars.get("VERBOSE"))
         self.DATE = base_env_vars.get("DATE")
-        self.S3_BUCKET = base_env_vars.get("S3_BUCKET")
+        self.S3_BUCKET = "ndp-preview-" + base_env_vars.get("S3_BUCKET")
         self.PROD_INSTANCE_NAME = base_env_vars.get("PROD_INSTANCE_NAME") or base_env_vars.get("CI_PROJECT_NAME")
         self.DB_NAME = base_env_vars.get("DB_NAME")
         self.ODOO_DEPENDS = base_env_vars.get("ODOO_DEPENDS")
@@ -53,7 +53,7 @@ class EnvironementConfig:
         self.S3_DUMP_SECRET = self.S3_DUMP_SECRET or "s3-ndp-scaleway"
         self.S3_DUMP_BUCKET = self.S3_DUMP_BUCKET or "ndp-production-backups"
         self.DATE = self.DATE or date.today().isoformat()
-        self.S3_BUCKET = self.S3_BUCKET or self.NESTOR_NAME
+        self.S3_BUCKET = "ndp-preview-" + (self.S3_BUCKET or self.NESTOR_NAME)
         self.DB_NAME = self.DB_NAME or self.PROD_INSTANCE_NAME
         self.ODOO_DEPENDS = self.ODOO_DEPENDS or "odoo-addons/common-modules,odoo-addons/community-addons"
         dump_path_fmt = "%(PROD_INSTANCE_NAME)s/journaliere/%(DB_NAME)s_%(DATE)s.dump"
