@@ -1,15 +1,18 @@
+import enum
+import os
 import re
 import subprocess
 import sys
 from functools import cached_property
 from typing import List, Union
 
-from yaml import load
+from yaml import dump, load
 
 try:
+    from yaml import CDumper as Dumper
     from yaml import CLoader as Loader
 except ImportError:
-    from yaml import Loader
+    from yaml import Dumper, Loader
 
 
 ######################################
@@ -181,7 +184,10 @@ class NestorOpt(NestorDesc):
         return NestorDescSet([self])
 
     def _key_hash(self):
-        return tuple(self.name)
+        name = self.name
+        if not name.startswith(NestorOpt._prefix):
+            name = NestorOpt._prefix + name
+        return name
 
     def __str__(self):
         name = self.name
@@ -285,6 +291,7 @@ class NestorInstance:
         self.name = name
         self.verbose = verbose
         self.filestore = NestorFileStoreCommand(self)
+        self.db = NestorDatabaseCmd(self)
 
     def __repr__(self):
         return "nestor[%s]" % self.name
@@ -311,6 +318,24 @@ class NestorInstance:
             self.edit(self.progressive_spec)
         self.progressive_spec = None
 
+    def save_spec(self, values: NestorDescSet, path_dir: str):
+        full_path = path_dir
+        if os.path.splitext(path_dir)[1] != "spec":
+            full_path = os.path.join(path_dir, self.name + ".spec")
+        with open(os.path.expanduser(full_path), "w+") as spec_file:
+            spec_file.write(values.to_str(True))
+        return full_path
+
+    @staticmethod
+    def new_from_yaml(path_spec, name=None):
+        ext = os.path.splitext(path_spec)[1]
+        if ext not in ["yaml", "yml"]:
+            raise ValueError(f"Le format [{ext}] du fichier {path_spec} n'est pas supporté")
+        name = name or os.path.splitext(os.path.basename(path_spec))[0]
+        inst = NestorInstance(name)
+        inst.direct_call(f"cat {path_spec} | nestor new {name} -")
+        return inst
+
     def exist(self):
         return not bool(self.wait(postgres=True, timeout=2))
 
@@ -373,7 +398,9 @@ class NestorInstance:
         verbose: bool = False,
     ) -> int:
 
-        return NestorDatabaseCmd(self).restore("s3", dump_path, s3_secret, bucket, set_password_to_all, verbose)
+        return self.db.restore(
+            NestorDatabaseCmd.DBCmdTarget.S3, dump_path, s3_secret, bucket, set_password_to_all, verbose
+        )
 
     def wait(self, up: bool = True, postgres: bool = False, timeout: int = None) -> int:
         opt = []
@@ -442,6 +469,17 @@ class NestorInstance:
     def spec(self) -> dict:
         return load(self.show("yaml"), Loader)["spec"]
 
+    @cached_property
+    def spec_values(self) -> NestorDescSet:
+        desc = NestorDescSet()
+        specs = self.show("spec").split(",spec")
+        key, value = specs[0].split("=")
+        desc.add(NestorOpt(key, value))
+        for opt in specs[1:]:
+            key, value = opt.split("=")
+            desc.add(NestorOpt("spec" + key, value))
+        return desc
+
     @cached_property
     def spec_text(self):
         return self.show("text")
@@ -465,15 +503,21 @@ class NestorDatabaseCmd:
     def __init__(self, inst: NestorInstance):
         self._inst = inst
 
+    class DBCmdTarget(enum.Enum):
+        LOCAL = "local"
+        S3 = "s3"
+
     def restore(
         self,
-        type: str,
+        type: Union[str, DBCmdTarget],
         dump_path: str,
         s3_secret: str = None,
         bucket: str = None,
         set_password_to_all: bool = False,
         verbose: bool = False,
     ) -> int:
+        if isinstance(type, NestorDatabaseCmd.DBCmdTarget):
+            type = type.value
         opt = [type, dump_path]
         if s3_secret:
             opt.append("--s3")
@@ -487,8 +531,26 @@ class NestorDatabaseCmd:
             opt.append("--verbose")
         return NestorCommand(self._inst, ["database", "restore"], *opt).call()
 
-    def dump(self):
-        raise NotImplementedError
+    def dump(
+        self,
+        type: Union[str, DBCmdTarget],
+        dump_path: str,
+        s3_secret: str = None,
+        bucket: str = None,
+        verbose: bool = False,
+    ):
+        if isinstance(type, NestorDatabaseCmd.DBCmdTarget):
+            type = type.value
+        opt = [str(type), dump_path]
+        if s3_secret:
+            opt.append("--s3")
+            opt.append(s3_secret)
+        if bucket:
+            opt.append("--bucket")
+            opt.append(bucket)
+        if verbose:
+            opt.append("--verbose")
+        return NestorCommand(self._inst, ["database", "dump"], *opt).call()
 
     def secret(self):
         raise NotImplementedError
@@ -503,27 +565,36 @@ class NestorFileStoreCommand:
 
     def enable_on_s3_spec(self, s3_secret: str, s3_bucket: str = None, values: NestorDescSet = None):
         assert s3_secret, "A s3 secret is required"
-        assert self.inst.version(values) <= 14, "V15 not supported yet"
+        assert self.inst.version(values) <= 15, "V15 not supported yet"
 
         s3_bucket = s3_bucket or self.inst.name
         specs = NestorPersistenceDesc(s3_secret, s3_bucket, enable_s3=True).to_set_opt()
         loads = (values["options.load"] or NestorOpt("options.load", "")).value.split(",")
 
-        if self.inst.version(values) == 14:
+        branch_name = str(float(self.inst.version(values)))
+        if self.inst.version(values) >= 14:
             path = "odoo/cloud-modules"  # New repo only for 14.0 for now
             if "odoo_filestore_s3" not in loads:
                 loads.append("odoo_filestore_s3")
+            if self.inst.version(values) == 15:
+                branch_name = "1.2.1"
         else:
             path = "kubernetes/odoo-modules"
             if "odoo_s3" not in loads:
                 loads.append("odoo_s3")
-        for idx in [""] + list(range(2, 20)):
+        for idx in list(range(2, 20)):
             opt = "sources.repositories.DEPOT_GIT%s.path" % idx
             if not values[opt]:
+                specs.add(NestorGitDesc(idx, path, branch_name).to_set_opt())
                 break
-        specs.add(NestorGitDesc(4, path, str(float(self.inst.version(values)))).to_set_opt())
-
-        specs.add(NestorOpt("options.load", ",".join(loads)))
+        else:
+            raise ValueError("Trop de DEPOT_GIT: superieur a 20")
+        if self.inst.version(values) <= 14:
+            specs.add(NestorOpt("options.load", ",".join(loads)))
+        else:
+            # les options sont separees par des espaces pour eviter le probleme de --set de nestor qui splitte sur
+            # les virgules
+            specs.add(NestorOpt("options.load", " ".join(loads)))
 
         return specs
 
@@ -552,6 +623,10 @@ class ScriptNestorInstance(NestorInstance):
     def __init__(self, name, verbose=False):
         super(ScriptNestorInstance, self).__init__(name, verbose)
 
+    @staticmethod
+    def from_inst(nestor_inst: NestorInstance):
+        return ScriptNestorInstance(nestor_inst.name, nestor_inst.verbose)
+
     def delete_and_exit_if_failed(self, result: int):
         self.delete_if_failed(result)
         if result:
